import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
//Extra imports.

import java.awt.Color;
import java.awt.event.*;
import java.awt.GridLayout;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JOptionPane;

/**
 * Class tha controls Pokemon actions in the battle field (PokeChessBoard).
 * <br /><br />
 * Classe que controla as ações dos pokémons no campo de batalha (PokeChessBoard).
 * 
 * @author Marcelo Schmitt 
 * @version 1.1 - 19/01/2014
 */
public class ActionController extends Actor implements ActionListener
{
    private JButton moveButton;
    private JButton attackButton;
    
    private JButton attack1Button;
    private JButton attack2Button;
    
    /**A versatile frame*/
    private JFrame actionFrame;
    
    /**The pokemon which we are dealing with.*/
    private Pokemon selectedPokemon;
    
    /**This variable(array) contains the selectedPokemon's attack methods.*/
    private ArrayList<Method> attackMethods;
    
    /**This variable contains the number of the attack method in the pokeMethods array.*/
    private int attackNumber;
    
    /**This variable is used to control the pokemon placing in the first two turns.*/
    private int pokeNumber;
    
    /**
     * Constructor of the ActionController class.
     * <br /><br />
     * Construtor da classe ActionController.
     */
    public ActionController(){
        actionFrame = new JFrame();
        this.pokeNumber = 0;
        this.attackNumber = 0;
    }
    
    /**
     * Build a frame JFrame to show the stats and the actions avaliable for the pokemon.
     * <br /><br />
     * Constroi uma janela JFrame para exibir o status e as ações disponíveis para o pokémon.
     * 
     * @param pokemon 
     *          the pokemon to have it's information displayed.
     */
    private void buildActionFrame(Pokemon pokemon){
        JLabel maxHitPointsLabel = new JLabel("Max hit points: " + pokemon.getMaxHitPoints());
        JLabel hitPointsLabel = new JLabel("Hit Points: " + pokemon.getHitPoints());
        JLabel typeLabel = new JLabel("Type: " + pokemon.getType());
        JLabel weaknessLabel = new JLabel("Weakness: " + pokemon.getWeakness());
        JLabel resistanceLabel = new JLabel("Resistance: " + pokemon.getResistance());
        
        JPanel statusPanel = new JPanel(new GridLayout(5,1));
        
        statusPanel.add(maxHitPointsLabel);
        statusPanel.add(hitPointsLabel);
        statusPanel.add(typeLabel);
        statusPanel.add(weaknessLabel);
        statusPanel.add(resistanceLabel);
        
        moveButton = new JButton("Move");
        moveButton.addActionListener(this);
        attackButton = new JButton("Attack");
        attackButton.addActionListener(this);
        
        moveButton.setEnabled(pokemon.isMoveAble());
        attackButton.setEnabled(pokemon.isAttackAble());
        PokeChessBoard board = (PokeChessBoard) getWorld();
        /*
         * O jogo começa no turno 0, por causa disso, o jogador que for o player 1 fará suas jogadas sempre em turnos pares: 0, 2, 4, etc..
         * Consequentemente o jogador que for o player 2 fará suas jogadas nos turnos ímpares: 1, 3, 5, etc..
         * Se o pokémon selecionado for do player 1 e o número do turno for ímpar, ou, 
         * se o pokémon selecionado for do player 2 e o número do turno for par, 
         * então o pokémon é de um jogador que não está em sua vez de jogar e devemos desabilitar os botões de ação.
         */
        if(  (pokemon.getOwner().getPlayerNumber() == 1 && board.getTurnNumber() % 2 != 0) 
          || (pokemon.getOwner().getPlayerNumber() == 2 && board.getTurnNumber() % 2 == 0)  ){
            moveButton.setEnabled(false);
            attackButton.setEnabled(false);
        }
        
        JPanel actionPanel = new JPanel(new GridLayout(2,1));
        actionPanel.add(moveButton);
        actionPanel.add(attackButton);
        
        JPanel mainPanel = new JPanel(new GridLayout(2,1));
        mainPanel.add(statusPanel);
        mainPanel.add(actionPanel);
        
        //Cleaning actionFrame.
        actionFrame.setVisible(false);
        actionFrame = null;
        //Creating a JFrame and displaying it.
        actionFrame = new JFrame(board.getPlayer(pokemon.getOwner().getPlayerNumber()).getName() + "'s " + pokemon.getName());       
        actionFrame.add(mainPanel);
        actionFrame.pack();
        //actionFrame.setLocation(350, 200);
        actionFrame.setLocationByPlatform(true);
        actionFrame.setSize(250, 175);
        actionFrame.setVisible(true);
        
        selectedPokemon = pokemon;
    }
    
    /**
     * Build a frame with the attack options for a pokemon.
     * <br /><br />
     * Constroi uma janela com as opções de ataque para um pokémon.
     */
    private void buildAttackFrame(){
        attack1Button = new JButton(attackMethods.get(0).getAnnotation(Attack.class).name());
        attack1Button.setToolTipText(attackMethods.get(0).getAnnotation(Attack.class).description());
        attack1Button.addActionListener(this);
        
        attack2Button = new JButton(attackMethods.get(1).getAnnotation(Attack.class).name());
        attack2Button.setToolTipText(attackMethods.get(1).getAnnotation(Attack.class).description());
        attack2Button.addActionListener(this);
        
        JPanel attackPanel = new JPanel(new GridLayout(2,1));
        attackPanel.add(attack1Button);
        attackPanel.add(attack2Button);
        
        //Cleaning actionFrame.
        actionFrame.setVisible(false);
        actionFrame = null; 
        
        actionFrame = new JFrame(new String(selectedPokemon.getName() + "'s attacks"));
        actionFrame.add(attackPanel);
        actionFrame.pack();
        //actionFrame.setLocation(350, 200);
        actionFrame.setLocationByPlatform(true);
        actionFrame.setSize(225, 100);
        actionFrame.setVisible(true);
    }
    
    /**
     * Calls the show...AttackRange mothods at Pokemon to show the places where an attack can reach.
     * <br /><br />
     * Chama os métodos do tipo show...AttackRange em Pokemon para mostrar quais casas um ataque pode alcançar.
     */
    private void showAttackRange() {
        switch(attackMethods.get(attackNumber).getAnnotation(Attack.class).rangeType()) {
            case NONE:
                return;
            case SELF:
                selectedPokemon.showSelfAttackRange();
                return;
            case MELEE:
                selectedPokemon.showMeleeAttackRange();
                return;
            case RANGED:
                selectedPokemon.showRangedAttackRange(attackMethods.get(attackNumber).getAnnotation(Attack.class).range());
                return;
            case BY_AREA:
                selectedPokemon.showByAreaAttackRange(attackMethods.get(attackNumber).getAnnotation(Attack.class).range());
                return;
            case IN_AREA:
                selectedPokemon.showInAreaAttackRange(attackMethods.get(attackNumber).getAnnotation(Attack.class).range());
                return;
        }
    }
    
    /**
     * Frees the memory space alocated to hold the objects whithing this ActionController.
     * <br /><br />
     * Libera o espaço de memória utilizado para guardar os objetos contidos neste ActionController.
     */
    @Override
    public void finalize(){
        actionFrame.setVisible(false);
        actionFrame = null;
        attackMethods = null;
        selectedPokemon = null;
    }
    
    /**
     * Act - do whatever the ActionController wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     * 
     * 
     * 
     */
    public void act() 
    {
        PokeChessBoard board = (PokeChessBoard) getWorld();
        MouseInfo mouseInfo = Greenfoot.getMouseInfo();
        
        //Turns 0 and 1 are pokémon placing turns, so we have a different game management at these turns.
        //Os turnos 0 e 1 são para colocar os pokémons, para isso temos um gerenciamento de jogo diferente nesses turnos.
        if(board.getTurnNumber() == 0){
            //Player 1 pokémon placing turn.
            Player p1 = board.getPlayer(1);
            if(Greenfoot.mouseMoved(null)) {
                setLocation(mouseInfo.getX(), mouseInfo.getY());
            }
            if(Greenfoot.mouseClicked(null)){
                if(getOneIntersectingObject(null) == null){
                    //Clicked on an empty place.
                    board.addObject(p1.getPokemonAt(pokeNumber), mouseInfo.getX(), mouseInfo.getY());
                    //Possivelmente haverá a necessidade de contar quantos turnos cada pokémon está em jogo.
                    //p1.getPokemonAt(pokeNumber).setFirstTurn(0);
                    pokeNumber++;
                    
                    if(pokeNumber == 3){
                        board.clearObjects(UnavaliableField.class);
                        board.nextTurn();
                        //Setting pokeNumber as 0 to start placing player 2's pokémons.
                        pokeNumber = 0;
                    }else{
                        this.setImage(p1.getPokemonAt(pokeNumber).getImage());
                    }
                } else{
                    Greenfoot.playSound("Pokemon program error.wav");
                    JOptionPane.showMessageDialog(null, "You can not place your pokemon here.", 
                                                    "Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
        } else if(board.getTurnNumber() == 1){
            //Player 2 pokémon placing turn.
            Player p2 = board.getPlayer(2);
            
            if(Greenfoot.mouseMoved(null)) {
                setLocation(mouseInfo.getX(), mouseInfo.getY());
            }
            if(Greenfoot.mouseClicked(null)){
                if(getOneIntersectingObject(null) == null){
                    //Clicked on an empty place.
                    board.addObject(p2.getPokemonAt(pokeNumber), mouseInfo.getX(), mouseInfo.getY());
                    //Possivelmente haverá a necessidade de contar quantos turnos cada pokémon está em jogo.
                    //p2.getPokemonAt(pokeNumber).setFirstTurn(1);
                    pokeNumber++;
                    if(pokeNumber == 3){
                        board.clearObjects(UnavaliableField.class);
                        board.nextTurn();
                        this.setLocation(0,0);
                        //Making this Actor "invisible".
                        this.setImage(new GreenfootImage(1,1));
                    }else{
                        this.setImage(p2.getPokemonAt(pokeNumber).getImage());
                    }
                } else{
                    Greenfoot.playSound("Pokemon program error.wav");
                    JOptionPane.showMessageDialog(null, "You can not place your pokemon here.", 
                                                    "Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
        } else{
            //If it's not turn 0 either 1.
            //Não é mais o primeiro nem o segundo turno.
            //Check if the mouse was clicked.
            //Confere se o mouse foi clicado.
            if(Greenfoot.mouseClicked(null)) {
                Object clickedObj = mouseInfo.getActor();
                if(clickedObj instanceof Pokemon){
                    Pokemon pokemon = (Pokemon) mouseInfo.getActor();
                    buildActionFrame(pokemon);
                } else if(clickedObj instanceof MoveField){
                    //Move the selected pokemon.
                    selectedPokemon.move(mouseInfo.getX(), mouseInfo.getY());
                } else if(clickedObj instanceof AttackField){
                    Pokemon targetPokemon = 
                        (Pokemon) getOneObjectAtOffset(mouseInfo.getX(), mouseInfo.getY(), Pokemon.class);
                    if(targetPokemon != null){
                        Object atReturn;
                        Object[] args = new Object[1];
                        args[0] = targetPokemon;
                        try{
                            //Here comes a call to a creepy method.
                            //invoke executa um método de um determinado objeto (um pokemon neste caso).
                            atReturn = attackMethods.get(attackNumber).invoke(selectedPokemon, args);
                        } catch(IllegalAccessException iacce){
                            JOptionPane.showMessageDialog(null, "IllegalAccessException");
                        } catch(IllegalArgumentException iarge){
                            JOptionPane.showMessageDialog(null, iarge.getMessage(), "IllegalArgumentException", JOptionPane.WARNING_MESSAGE);
                        } catch(NullPointerException npe){
                            JOptionPane.showMessageDialog(null, "NullPointerException");
                        } catch(InvocationTargetException ite){
                            JOptionPane.showMessageDialog(null, ite.getTargetException().getMessage(), 
                                                "Invalid Target", JOptionPane.WARNING_MESSAGE);
                        } catch(ExceptionInInitializerError eiie){
                            JOptionPane.showMessageDialog(null, "ExceptionInInitializerError");
                        }
                        atReturn = null;
                        args = null;
                    }
                }
                clickedObj = null;
                board.clearObjects(UnavaliableField.class);
                board.clearObjects(MoveField.class);
                board.clearObjects(AttackField.class);
            }
        }
    }
    
    /**
     * Method that handles actionFrame button's events.
     * <br /><br />
     * Método que trata os eventos de botões da janela actionFrame.
     */
    public void actionPerformed (ActionEvent e){
        Object source = e.getSource();
        if(source == moveButton){
            selectedPokemon.showMoveField();
            actionFrame.setVisible(false);
        } else if(source == attackButton){
            //Here begins the reflection =]. Here we take an array of our pokemon methods.
            //Aqui começa o trabalho com reflectino =]. Pegamos um array com os métodos do nosso pokémon.
            Method[] pokeMethods = selectedPokemon.getClass().getMethods();
            attackMethods = new ArrayList<Method>();
            for (int i=0; i < pokeMethods.length; i++) {
                //Se o método for um método anotado com a anotação Attack então o adicionamos à lista attackMethods.
                if (pokeMethods[i].getAnnotation(Attack.class) != null) {
                    attackMethods.add(pokeMethods[i]);
                }
            }
            this.buildAttackFrame();
            /*
             * TODO
             * Ao envez de todo este trabalho com reflection que me restringe a ter que escrever um método novo a cada ataque 
             * que um pokémon pussui - mesmo se eu tiver dois ataqueis iguais em dois pokémons diferentes - talvez possa ser 
             * utilizado do design pattern Command mantendo uma lista de ataques em cada pokémon.
             * Isso tornaria mais facil a criação, reutilização e alteração de ataques e também tornaria possível adicionar e remover 
             * ataques de pokémons.
             */
        } else if(source == attack1Button){
            attackNumber = 0;
            showAttackRange();
            actionFrame.setVisible(false);
        } else if(source == attack2Button){
            attackNumber = 1;
            showAttackRange();
            actionFrame.setVisible(false);
        }
    }
}