import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

import java.awt.BorderLayout;
import javax.swing.DefaultComboBoxModel;

import java.awt.event.*;
import java.awt.GridLayout;
import java.awt.Font;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import javax.swing.JOptionPane;
import javax.swing.JTextField;

/**
 * Class that build the main menu.
 * 
 * Classe que constroi o menu principal.
 * 
 * @author Marcelo Schmitt 
 * @version 1.5 - 10/12/2013
 */
public class MainMenu extends World implements ActionListener
{
    JFrame startMenu = new JFrame();
    
    private MenuButton startButton;
    private MenuButton rulesButton;
    private MenuButton creditsButton;
    private MenuButton backButton;
    
    private JComboBox p1PokeSlot1;
    private JComboBox p1PokeSlot2;
    private JComboBox p1PokeSlot3;
    
    private JComboBox p2PokeSlot1;
    private JComboBox p2PokeSlot2;
    private JComboBox p2PokeSlot3;
    
    private JButton startBattleButton;
    
    private JTextField p1NameTxt;
    private JTextField p2NameTxt;
    
    /**POKEMONARRAY is an array with the names of all avaliable pokemon in game.*/
    private final String[] POKEMONARRAY;
    
    /**
     * Constructor for MainMenu objects.
     * 
     * Construtor para o Menu.
     */
    public MainMenu()
    {    
        //Create a new World with 800x600 cells with a cell size of 1x1 pixel. The size of the main menu image.
        //Cria um World com 800x600 células em que cada célula ocupa 1x1 pixel. Esta é a resolução da imagem do menu.
        super(800, 600, 1); 
        
        //Creating MenuButton actors.
        startButton = new MenuButton("Start", 250, 75);
        this.addObject(startButton, 400, 200);
        rulesButton = new MenuButton("Game Rules", 425, 75);
        this.addObject(rulesButton, 400, 300);
        creditsButton = new MenuButton("Credits", 300, 75);
        this.addObject(creditsButton, 400, 400);
        backButton = new MenuButton("Back to Menu", 450, 75);
        //backButton is not added to the main menu initially.
        
        //This array holds the pokemon names that show on the JComboBox objects.
        //Este array guarda os nomes dos pokémons que aparecem nos JComboBox.
        this.POKEMONARRAY = new String[5];
        this.POKEMONARRAY[0] = "Bulbasaur";
        this.POKEMONARRAY[1] = "Charmander";
        this.POKEMONARRAY[2] = "Squirtle";
        this.POKEMONARRAY[3] = "Pikachu";
        this.POKEMONARRAY[4] = "Geodude";
        
        //TODO Fazer um carregamento mais eficiente de pokémons. Talvez deixar os nomes em um XML.
        //TODO It would be good to have a background music for the main menu.
    }
    
    /**
     * Returns a Pokemon object acording to the argument number.
     * * 0 returns Bulbasaur,
     * * 1 retruns Charmander,
     * * 2 returns Squirtle,
     * * 3 returns Pikachu,
     * * 4 returns Geodude,
     * * Any other number returns Bulbasaur.
     * 
     * Retorna um pokémon de acordo com o número do argumento.
     * * 0 retorna Bulbasaur,
     * * 1 retorna Charmander,
     * * 2 retorna Squirtle,
     * * 3 retorna Pikachu,
     * * 4 retorna Geodude,
     * * Qualquer outro número retorna Bulbasaur.
     * 
     * @param pokemonNumber the number of the wanted pokemon.
     * @return a specific Pokemon objetc.
     */
    public Pokemon getSelectedPokemon (int pokemonNumber){
        switch(pokemonNumber){
            case 0:
                return new Bulbasaur();
            case 1:
                return new Charmander();
            case 2:
                return new Squirtle();
            case 3:
                return new Pikachu();
            case 4:
                return new Geodude();
            default:
                return new Bulbasaur();
        }
    }
    
    /**
     * Handle the clicks on the menu buttons.
     * 
     * Trata os cliques nos botões do menu.
     */
    public void act(){
        if(Greenfoot.mouseClicked(startButton)){
            Greenfoot.playSound("Pokemon Menu Command.wav");
            
            p1NameTxt = new JTextField("Player1's name", 20);
            p2NameTxt = new JTextField("Player2's name", 20);
            
            JPanel p1NamePanel = new JPanel(); //Creates a new JPanel with a double buffer and a flow layout.
            p1NamePanel.add(new JLabel("Name:"));
            p1NamePanel.add(p1NameTxt);
            
            JPanel p2NamePanel = new JPanel(); //Creates a new JPanel with a double buffer and a flow layout.
            p2NamePanel.add(new JLabel("Name:"));
            p2NamePanel.add(p2NameTxt);
            
            //Creating JComboBox components for player1.{
            DefaultComboBoxModel p1PokeSlot1Model = new DefaultComboBoxModel(POKEMONARRAY);
            p1PokeSlot1 = new JComboBox(p1PokeSlot1Model);
            DefaultComboBoxModel p1PokeSlot2Model = new DefaultComboBoxModel(POKEMONARRAY);
            p1PokeSlot2 = new JComboBox(p1PokeSlot2Model);
            DefaultComboBoxModel p1PokeSlot3Model = new DefaultComboBoxModel(POKEMONARRAY);
            p1PokeSlot3 = new JComboBox(p1PokeSlot3Model);
            //Adding (this) ItemListener to the JComboBoxes.
            //p1PokeSlot1.addItemListener(this);
            //p1PokeSlot2.addItemListener(this);
            //p1PokeSlot3.addItemListener(this);
            //Seting JComboBoxes as non editable.
            p1PokeSlot1.setEditable(false);
            p1PokeSlot2.setEditable(false);
            p1PokeSlot3.setEditable(false);
            //}
            //Creating JComboBox components for player2.{
            DefaultComboBoxModel p2PokeSlot1Model = new DefaultComboBoxModel(POKEMONARRAY);
            p2PokeSlot1 = new JComboBox(p2PokeSlot1Model);
            DefaultComboBoxModel p2PokeSlot2Model = new DefaultComboBoxModel(POKEMONARRAY);
            p2PokeSlot2 = new JComboBox(p2PokeSlot2Model);
            DefaultComboBoxModel p2PokeSlot3Model = new DefaultComboBoxModel(POKEMONARRAY);
            p2PokeSlot3 = new JComboBox(p2PokeSlot3Model);
            //Adding (this) ItemListener to the JComboBoxes.
            //p2PokeSlot1.addItemListener(this);
            //p2PokeSlot2.addItemListener(this);
            //p2PokeSlot3.addItemListener(this);
            //Seting JComboBoxes as non editable.
            p2PokeSlot1.setEditable(false);
            p2PokeSlot2.setEditable(false);
            p2PokeSlot3.setEditable(false);
            //}
            
            JPanel p1PokemonSlotPanel = new JPanel(new GridLayout(3,1));
            JPanel p2PokemonSlotPanel = new JPanel(new GridLayout(3,1));
            
            p1PokemonSlotPanel.add(p1PokeSlot1);
            p1PokemonSlotPanel.add(p1PokeSlot2);
            p1PokemonSlotPanel.add(p1PokeSlot3);
            
            p2PokemonSlotPanel.add(p2PokeSlot1);
            p2PokemonSlotPanel.add(p2PokeSlot2);
            p2PokemonSlotPanel.add(p2PokeSlot3);
            
            JLabel p1Label = new JLabel("Player 1");
            JLabel p2Label = new JLabel("Player 2");
            
            JPanel p1Panel = new JPanel(new GridLayout(3,1));
            JPanel p2Panel = new JPanel(new GridLayout(3,1));
            
            p1Panel.add(p1Label);
            p1Panel.add(p1NamePanel);
            p1Panel.add(p1PokemonSlotPanel);
            
            p2Panel.add(p2Label);
            p2Panel.add(p2NamePanel);
            p2Panel.add(p2PokemonSlotPanel);
            
            startMenu.setVisible(false);
            startMenu = null;
            startMenu = new JFrame("Start Menu");
            startMenu.add(p1Panel, BorderLayout.WEST);
            startMenu.add(p2Panel, BorderLayout.EAST);
            
            startBattleButton = new JButton("Start Battle");
            startBattleButton.addActionListener(this);
            startMenu.add(startBattleButton, BorderLayout.SOUTH);
            
            startMenu.pack();
            startMenu.setLocation(250, 200);
            startMenu.setVisible(true);
        }
        if(Greenfoot.mouseClicked(rulesButton)){
            Greenfoot.playSound("Pokemon Menu Command.wav");
            
            this.removeObject(startButton);
            this.removeObject(rulesButton);
            this.removeObject(creditsButton);
            
            GreenfootImage rulesText = new GreenfootImage(800, 500);
            //Making the background whiter to draw the game rules.
            rulesText.setColor(new Color(255, 255, 255, 200));
            rulesText.fill();
            //Setting font, color and style to draw the game rules.
            Font font = rulesText.getFont();
            font = font.deriveFont(Font.BOLD, 16.0F);
            rulesText.setFont(font);
            rulesText.setColor(Color.BLACK);
            //Drawing the game rules.
            //Desenhando as regras do jogo.
            //TODO make a StringBuffer or StringBuilder to write this in a better way.
            //drawString desenha uma String dentro de uma imagem.
            rulesText.drawString("This is a two player game. ", 50, 50);
            rulesText.drawString("In this game you can simulate a multi-pokémon arena battle " +
                "within a chess board (a 8x8 arena).", 50, 75);
            rulesText.drawString("* Press enter to pass turn or back to main menu.", 50, 100);
            rulesText.drawString("* Click on the pokémons to open their action menu.", 50, 125);
            rulesText.drawString("Pokémon TCG adapted rules:", 50, 175);
            rulesText.drawString("Weakness and Resistance", 50, 225);
            rulesText.drawString("If a pokémon takes damage from a pokémon of its weakness " + 
                "type it takes double damage.", 50, 250);
            rulesText.drawString("If a pokémon takes damage from a pokémon of its resistance " + 
                "type it takes 3 less damage.", 50, 275);
            rulesText.drawString("Special Conditions", 50, 325);
            rulesText.drawString("Paralyzed (The only condition implemented untill now)",50, 350);
            rulesText.drawString("If a Pokémon is Paralyzed, it cannot attack or move.", 50, 375);
            rulesText.drawString("Paralysis condition is removed during the in-between " + 
                "turns phase if your Pokémon ", 50, 400);
            rulesText.drawString("was Paralyzed since the beginning of your last turn.", 50, 425);
            
            GreenfootImage rulesBackGround = this.getBackground();
            rulesBackGround.drawImage(rulesText, 0, 0);
            this.setBackground(rulesBackGround);
            
            //When we went to the rules menu we will need a back button.
            this.addObject(backButton, 400, 550);
        }
        if(Greenfoot.mouseClicked(creditsButton)){
            Greenfoot.playSound("Pokemon Menu Command.wav");
            
            this.removeObject(startButton);
            this.removeObject(rulesButton);
            this.removeObject(creditsButton);
            
            GreenfootImage creditsText = new GreenfootImage(800, 500);
            //Making the background whiter to draw the game rules.
            creditsText.setColor(new Color(255, 255, 255, 200));
            creditsText.fill();
            //Setting font color and style to draw the game rules.
            Font font = creditsText.getFont();
            font = font.deriveFont(Font.BOLD, 16.0F);
            creditsText.setFont(font);
            creditsText.setColor(Color.BLACK);
            //Drawing the game credits.
            //TODO make aStringBuffer or StringBuilder to write this in a better way.
            creditsText.drawString("PokéChess 1.5 credits", 50, 50);
            creditsText.drawString("Rules adapted from Pokémon Trading Card Game (TCG)", 50, 100);
            creditsText.drawString("http://assets2.pokemon.com/assets/cms/pdf/tcg/rulebooks/" + 
                "bw_next_destinies_rulebook.pdf", 50, 125);
            creditsText.drawString("Pokémon images from PokéDex Versão 1.8", 50, 175);
            creditsText.drawString("Fond at: http://www.baixaki.com.br/site/dwnld46010.htm?Btn=2", 50, 200);
            creditsText.drawString("Pokémon attacks from pokebeach website", 50, 250);
            creditsText.drawString("http://pokebeach.com/tcg/base-set/scans and", 50, 275);
            creditsText.drawString("http://pokebeach.com/tcg/fossil/scans", 50, 300);
            creditsText.drawString("Sounds and main menu background form Angels' Pokémon Theme", 50, 350);
            creditsText.drawString("Web Site: http://comnet.ca/~angel002/angelsdesktopthemes", 50, 375);
            creditsText.drawString("Found at: http://pokemon-theme.softonic.com", 50, 400);
            creditsText.drawString("Programming", 50, 450);
            creditsText.drawString("Marcelo S.", 50, 475);
            
            //Thank you for playing. //And also thank you for reading.
            creditsText.drawString("Thank you for playing", 575, 475);
            
            GreenfootImage creditsBackGround = this.getBackground();
            creditsBackGround.drawImage(creditsText, 0, 0);
            this.setBackground(creditsBackGround);
            
            //When we went to the credits menu we will need a back button.
            this.addObject(backButton, 400, 550);
        }
        if(Greenfoot.mouseClicked(backButton)){
            Greenfoot.playSound("Pokemon Menu Command.wav");
            //Let's rebuild the original MainMenu.
            this.removeObject(backButton);
            this.setBackground("Pokemon-wallpaper.jpg");
            this.addObject(startButton, 400, 200);
            this.addObject(rulesButton, 400, 300);
            this.addObject(creditsButton, 400, 400);
        }
    }
    
    /**
     * Handle the events generated by the buttons on the strat menu frame.
     * 
     * Trata os eventos gerados pelos botões do strat menu.
     */
    public void actionPerformed(ActionEvent e){
        //Handle the startBattleButton at the startMenu.
        if(e.getSource() == startBattleButton){
            JOptionPane.showMessageDialog(null, "start battle");
            startMenu.setVisible(false);
            startMenu = null;
            
            //Creating player1 and adding he's/her's pokemons.
            Player player1 = new Player(p1NameTxt.getText(), 1);
            player1.addPokemon(getSelectedPokemon(p1PokeSlot1.getSelectedIndex()));
            player1.addPokemon(getSelectedPokemon(p1PokeSlot2.getSelectedIndex()));
            player1.addPokemon(getSelectedPokemon(p1PokeSlot3.getSelectedIndex()));
            
            //Creating player2 and adding he's/her's pokemons.
            Player player2 = new Player(p2NameTxt.getText(), 2);
            player2.addPokemon(getSelectedPokemon(p2PokeSlot1.getSelectedIndex()));
            player2.addPokemon(getSelectedPokemon(p2PokeSlot2.getSelectedIndex()));
            player2.addPokemon(getSelectedPokemon(p2PokeSlot3.getSelectedIndex()));
            
            Greenfoot.playSound("Pokemon Open Program.wav");
            Greenfoot.setWorld(new PokeChessBoard(player1, player2));
        }
    }
    //Obs.: If you want to handle the JComboBoxes events use make this class implement ItemListener.
}