import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Class that represents a player.
 * <br /><br />
 * Classe que representa um jogador.
 * 
 * @author Marcelo Schmitt 
 * @version 1.1 - 19/01/2014
 */
public class Player extends Actor
{
    private String name;
    private Pokemon[] pokemons;
    private int playerNumber;
    
    /**
     * Creates a player with the specified name and number.
     * <br /><br />
     * Cria um jogador com o nome e número especificados.
     * 
     * @param name 
     *          the palyer's name
     * @param playerNumber 
     *          the player's number
     */
    public Player (String name, int playerNumber){
        this.name = name;
        this.playerNumber = playerNumber;
        this.pokemons = new Pokemon[3];
    }
    
    /**
     * Returns the player's name.
     * <br /><br />
     * Retorna o nome do jogador.
     * 
     * @return a String containing the Player's name
     */
    public String getName(){
        return this.name;
    }
    
    /**
     * Add a pokemon to the Player pokemon array in the next empty array position.
     * <br /><br />
     * Adiciona um pokémon ao array de pokémons do jogador no próximo elemento vazio do array.
     * 
     * @param pokemon 
     *          the pokemon to be added
     */
    public void addPokemon(Pokemon pokemon){
        for(int i=0; i < pokemons.length; i++){
            if(this.pokemons[i] == null){
                //Player has an empty pokemon slot.
                pokemon.setOwner(this);
                this.pokemons[i] = pokemon;
                return;
            }
        }
        //TODO If execution reach this point means that all pokemon slots are full. Display a warning/error message.
    }
    
    /**
     * Get a pokemon from the Player pokemon array.
     * <br /><br />
     * Obtem um pokémon do array de pokémons do jogador.
     * 
     * @param index 
     *          the index whitin the pokémon is
     * 
     * @return The Pokemon at the given index position. null if the given index is lower than 0 or higher than 2.
     */
    public Pokemon getPokemonAt(int index){
        //Testing for array bounds
        if(index >= 0 && index < 3){
            return pokemons[index];
        } else{
            return null;
        }
    }
    
    /**
     * Get the player's number.
     * <br /><br />
     * Obtem o número do jogador.
     * 
     * @return an int value indicatin this player's number
     */
    public int getPlayerNumber(){
        return this.playerNumber;
    }
    
    /**
     * Act - do whatever the Player wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        // Add your action code here.
    }    
}
