import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
//import java.awt.event.MouseAdapter;
//import java.awt.event.*;
import javax.swing.JOptionPane;
import java.util.Random;

/**
 * Write a description of class PokeChessBoard here.
 * 
 * @author Marcelo Schmitt   
 * @version 0.0.1 - 15/02/2012
 */
public class PokeChessBoard extends World
{
    private Player player1;
    private Player player2;
    
    /*
     * Moving to the Bench removes all Special Conditions. The only Special Conditions that 
     * prevent Pokémon from retreating are Asleep and Paralyzed. Since Asleep, Confused, and Paralyzed 
     * all rotate the Pokémon card, whichever one happened last to the Pokémon is the only one that is
     * still in efect. Since Poisoned and Burned use counters, those don’t afect other Special Conditions. 
     * An unfortunate Pokémon could be Poisoned, Burned, and Paralyzed all at the same time!
     */
    
    private int turnNumber;
    
    private ActionController actionController;
    
    /**
     * Constructor for objects of class PokeChessBoard.
     */
    public PokeChessBoard(Player player1, Player player2)
    {    
        // Create a new world with 8x8 cells with a cell size of 75x75 pixels.
        super(8, 8, 75);
        //Seting up instace variables.
        turnNumber = 0;
        this.player1 = player1;
        this.player2 = player2;
        //Initializing an ActionController to handle game's events.
        actionController = new ActionController();
        addObject(actionController, 0, 0);
        
        //TODO comment
        actionController.setImage(player1.getPokemonAt(0).getImage());
        //TODO comment
        for(int x=0; x <= 7; x++){
            for(int y=0; y <= 5; y++){
                addObject(new UnavaliableField(), x, y);
            }
        }
    }
    
    //static method filpCoin
    /**
     * Return
     * A random boolean value.
     * By default I will consider return True as Head and return False as Tail.
     */
    public static boolean flipCoin(){
        Random random = new Random();
        return random.nextBoolean();
    }
    
    /**
     * Return the game turn. It starts with 0 and increases for each time a player passes his/her turn.
     * For instance: 
     * Player1's first turn (which is turn 0) is his/her pokemon placing turn.
     * Player2's first turn (which is turn 1) is his/her pokemon placing turn.
     * Player1's second turn (which is trun 2) is his/her first pokemon action turn.
     * And so on... .
     */
    public int getTurnNumber(){
        return this.turnNumber;
    }
    
    /**
     * Go to the next game turn and do suitable tasks related to it, as:
     * * Support for player2 pokemon placing turn (trun 1);
     * * Handle pokemon satates;
     * * Handle pokemon special conditions;
     * * Display whose player is going to play the next turn.
     */
    public void nextTurn(){
        this.turnNumber++;
        
        //TODO comment
        if(this.turnNumber == 1){
            for(int x=0; x <= 7; x++){
                for(int y=2; y <= 7; y++){
                    addObject(new UnavaliableField(), x, y);
                }
            }
            actionController.setImage(player2.getPokemonAt(0).getImage());
        }else{
            this.clearObjects(UnavaliableField.class);
        }
        
        //Sets true for movement and attack for all the pokemons on board.//TODO recomment
        List<Pokemon> pokemonsOnBoard = getObjects(Pokemon.class);
        for(int i=0; i < pokemonsOnBoard.size(); i++){
            Pokemon pokemon = pokemonsOnBoard.get(i);
            pokemon.setMoveAble(true);
            pokemon.setAttackAble(true);
            
            //Implementation for special conditions and poison efects.
            
            if(pokemon.isPoisoned()){
                //
                if(pokemon.isDamageImmune()){
                    //
                    pokemon.setDamageImmune(false);
                    pokemon.setDamageTaken(1, null);
                    pokemon.setDamageImmune(true);
                } else{
                    pokemon.setDamageTaken(1, null);
                }
            }
            if(pokemon.isBurned()){
                //
                if(pokemon.isDamageImmune()){
                    pokemon.setDamageImmune(false);
                    pokemon.setDamageTaken((flipCoin()) ? 0 : 2, null);
                    pokemon.setDamageImmune(true);
                } else{
                    pokemon.setDamageTaken(1, null);
                }
            }
            if(pokemon.isAsleep()){
                if(flipCoin()){
                    pokemon.setAsleep(false);
                } else{
                    pokemon.setMoveAble(false);
                    pokemon.setAttackAble(false);
                }
            }
            if(pokemon.isParalyzed()){
                //
                pokemon.setMoveAble(false);
                pokemon.setAttackAble(false);
                if( (pokemon.getOwner().getPlayerNumber() == 1 && turnNumber % 2 != 0) ||
                    (pokemon.getOwner().getPlayerNumber() == 2 && turnNumber % 2 == 0) ){
                    //
                    pokemon.setParalyzed(false);
                }
                //TODO 
            }
            
            if( (pokemon.getOwner().getPlayerNumber() == 1 && turnNumber % 2 == 0) ||
                (pokemon.getOwner().getPlayerNumber() == 2 && turnNumber % 2 != 0) ){
                pokemon.setDamageImmune(false);
            }
            
        }
        
        
        
        //TODO comment
        if(turnNumber%2 == 0){
            JOptionPane.showMessageDialog(null, player1.getName() + "'s turn!", 
                                            "Turn passed", JOptionPane.INFORMATION_MESSAGE);
        } else{
            JOptionPane.showMessageDialog(null, player2.getName() + "'s turn!", 
                                            "Turn passed", JOptionPane.INFORMATION_MESSAGE);
        }
        
    }
    
    /**
     * 
     * Return:
     * The player2 if 2 is passed as argument, the player1 for any other argument number.
     */
    public Player getPlayer(int playerNumber){
        return (playerNumber == 2) ? this.player2 : this.player1;
    }
    
    /**
     *     Get all the objects in the world, or all the objects of a particular class and remove them.
     *     If a class is specified as a parameter, only objects of that class (or its subclasses) will be removed.
     *     It's the combination of the World class method getObjects with removeObjects.
     *     
     *     Parameters:
     *     clazz - Class of objects to look for and remove ('null' will find all objects).
     */
    public void clearObjects(Class clazz){
        List<Object> objectList = this.getObjects(clazz);
        this.removeObjects(objectList);
    }
    
    /**
     * Checks if a player (or both) got all pokemon knocked out.
     * If so, return true, else, false.
     */
    private boolean endGameCheck(){
        boolean endGame = false;
        boolean p1Lose = true;
        for(int i=0; i < 3; i++){
            p1Lose &= player1.getPokemonAt(i).isKnockedOut();
        }
        boolean p2Lose = true;
        for(int i=0; i < 3; i++){
            p2Lose &= player2.getPokemonAt(i).isKnockedOut();
        }
        if(p1Lose && p2Lose){
            //Draw.
            JOptionPane.showMessageDialog(null, "Battle draw.", "Draw", JOptionPane.INFORMATION_MESSAGE);
            endGame = true;
        } else if(p1Lose){
            //Player 1 lost. Player 2 won.
            JOptionPane.showMessageDialog(null, "Player 2: " + player2.getName() + " won the battle!", 
                                            "Win", JOptionPane.INFORMATION_MESSAGE);
            endGame = true;
        } else if(p2Lose){
            //Player 2 lost. Player 1 won.
            JOptionPane.showMessageDialog(null, "Player 1: " + player1.getName() + " won the battle!", 
                                            "Win", JOptionPane.INFORMATION_MESSAGE);
            endGame = true;
        }
        return endGame;
    }
    
    public void endGame(){
        Greenfoot.playSound("Pokemon close program.wav");
        //actionController.finalize();
        this.clearObjects(UnavaliableField.class);
        this.clearObjects(Pokemon.class);
        removeObject(actionController);
        actionController = null;
        Greenfoot.setWorld(new MainMenu());
    }
    
    public void act(){
        //
        if(endGameCheck()){
            //
            endGame();
        }
        if(Greenfoot.isKeyDown("enter")){
            Object[] options = {"Pass Turn", "Quit Game", "Cancel"};
            int option;
            option = JOptionPane.showOptionDialog(null, "Quat do you want to do?", "Game Menu", JOptionPane.YES_NO_CANCEL_OPTION, 
                JOptionPane.QUESTION_MESSAGE, null, options, 2);
            if(option == 0){
                Greenfoot.playSound("Pokemon Asterisk.wav");
                nextTurn();
            } else if(option == 1){
                if(JOptionPane.showConfirmDialog(null, "Quit to main menu?", "Exit game?", JOptionPane.OK_CANCEL_OPTION, 
                    JOptionPane.QUESTION_MESSAGE) == 0){
                        //TODO comment
                        endGame();
                }
            }
        }
    }
}
