import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
//
//import javax.swing.JOptionPane;

/**
 * Class which represents a pokemon.
 * This class contains the atributes (name, type, hit points, etc.) and
 * caracteristics (ability to move, show where can attack and absorb damage) which each pokemon have.
 * <br /><br />
 * Classe que representa um pokémon.
 * Esta classe contem os atributos (nome, tipe, pontos de vida, etc.) e
 * caractarísticas (abilidade de se mover, mostrar onde pode atacar e abserver dano) que cada pokémon tem.
 * 
 * @author Marcelo Schmitt 
 * @version 1.1 - 19/01/2014
 */
public abstract class Pokemon extends Actor
{
    //Pokemon's intrinsic atributes.
    //Atributos intríncecas dos pokémons.
    private String name;
    private PokemonType type;
    private final int maxHitPoints;
    private int hitPoints;
    private PokemonType weakness;
    private PokemonType resistance;
    //Game variables.
    //Variáveis do jogo.
    private Player owner;
    private int retreatCost;
    //private int firstTurn;
    private boolean moveAble; //Tells weather the pokemon can move. Diz se o pokémon pode se mover ou não.
    //private boolean flyAble;
    private boolean attackAble; //Tells weather the pokemon can attack. Diz se o pokémon pode atacar ou não.
    //Special condition atributes{
    //Atributos de condições especiáis
    private boolean asleep;
    private boolean burned;
    private boolean confused;
    private boolean paralyzed;
    private boolean poisoned;
    //}
    private boolean damageImmune;
    //private boolean effectImmune;
    private boolean knockedOut;
    
    /**
     * Creates a pokemon with full health.
     * <br /><br />
     * Cria um pokémon com vida cheia.
     * 
     * @param name 
     *          the pokemon's name
     * @param type 
     *          the pokemon's element type
     * @param maxHitPoints 
     *          the pokemon's maximum and current hit points
     * @param weakness 
     *          the elemental weakness of this pokemon
     * @param resistance 
     *          the elemental resistance of this pokemon
     * @param retreatCost 
     *          this pokemon retreat cost
     */
    public Pokemon(String name, PokemonType type, int maxHitPoints, PokemonType weakness, PokemonType resistance, 
                    int retreatCost){
        this(name, type, maxHitPoints, maxHitPoints, weakness, resistance, retreatCost);//Initializes the pokémon with full health.
    }
    
    /**
     * Creates a pokemon.
     * <br /><br />
     * Cria um pokémon.
     * 
     * @param name 
     *          the pokemon's name
     * @param type 
     *          the pokemon's element type
     * @param maxHitPoints 
     *          the pokemon's maximum hit points
     * @param currentHitPoints 
     *          the pokemon's current hit points
     * @param weakness 
     *          the elemental weakness of this pokemon
     * @param resistance 
     *          the elemental resistance of this pokemon
     * @param retreatCost 
     *          this pokemon retreat cost
     */
    protected Pokemon(String name, PokemonType type, int maxHitPoints, int currentHitPoints, PokemonType weakness, PokemonType resistance, 
                    int retreatCost){
        this.name = name;
        this.type = type;
        this.maxHitPoints = maxHitPoints;
        this.hitPoints = currentHitPoints;
        this.weakness = weakness;
        this.resistance = resistance;
        this.retreatCost = retreatCost;
    }
    
    /**
     * Removes all this pokemon's special conditions.
     * <br /><br />
     * Remove todas as condições especiais deste pokémon.
     */
    public void setConditionsClear () {
        moveAble = true; //Initialy every pokemon can move.
        attackAble = true;
        
        asleep = false;
        burned = false;
        confused = false;
        paralyzed = false;
        poisoned = false;
        
        damageImmune = false;
        knockedOut = false;
    }
    
    //State methods
    /**
     * Returns this pokemon's name.
     * <br /><br />
     * Retorna o nome deste pokémon.
     * 
     * @return a String containing this pokemon's name
     */
    public String getName(){
        return this.name;
    }
    /**
     * Returns this pokemon's element type.
     * <br /><br />
     * Retorna o tipo elemental deste pokémon.
     * 
     * @return a PokemonType with the type of this pokemon
     */
    public PokemonType getType(){
        return this.type;
    }
    /**
     * Returns the amount of current hitpoints for this pokemon.
     * <br /><br />
     * Retorna a quantidade de pontos de vida para este pokémon.
     * 
     * @return an integer with this pokemon's hitpoints
     */
    public int getHitPoints(){
        return this.hitPoints;
    }
    /**
     * Returns the maximum amount of hitpoints for this pokemon.
     * <br /><br />
     * Retorna a quantidade máxima de pontos de vida para este pokémon.
     * 
     * @return an integer with this pokemon's maximun hitpoints
     */
    public int getMaxHitPoints(){
        return this.maxHitPoints;
    }
    /**
     * Returns this pokemon's element weakness.
     * <br /><br />
     * Retorna a fraquesa deste pokémon.
     * 
     * @return a PokemonType with the weakness of this pokemon
     */
    public PokemonType getWeakness(){
        return this.weakness;
    }
    /**
     * Returns this pokemon's element resistance.
     * <br /><br />
     * Retorna a resistência deste pokémon.
     * 
     * @return a PokemonType with the resistance of this pokemon
     */
    public PokemonType getResistance(){
        return this.resistance;
    }
    /**
     * Returns this pokemon's owner.
     * <br /><br />
     * Retorna o dono deste pokémon.
     * 
     * @return returns the Player which owns this pokemon
     */
    public Player getOwner(){
        return this.owner;
    }
    /**
     * Defines a owner for this pokemon.
     * <br /><br />
     * Define um dono para este pokémon.
     * 
     * @param player 
     *          the player that owns this pokemon
     */
    public void setOwner(Player player){
        this.owner = player;
    }
    /**
     * Defines the retreat cost for this pokemon.
     * <br /><br />
     * Define o custo de recúo para este pokémon.
     * 
     * @param retreatCost 
     *          the retreat cost for this pokemon
     */
    public void setRetreatCost(int retreatCost) {
        this.retreatCost = retreatCost;
    }
    /**
     * Returns this pokemon's retreat cost.
     * <br /><br />
     * Retorna o custo de recúo deste pokémon.
     * 
     * @return an integer indicating this pokemon's retreat cost
     */
    public int getRetreatCost() {
        return this.retreatCost;
    }
    /**
     * Tells whether this pokemon can move.
     * <br /><br />
     * Diz se este pokémon pode se mover.
     * 
     * @return true if this pokemon can move
     */
    public boolean isMoveAble(){
        return this.moveAble;
    }
    /**
     * Defines whether this pokemon can move.
     * <br /><br />
     * Define se este pokémon pode se mover.
     * 
     * @param moveAble 
     *          a boolean value indicating if this pokemon can move
     */
    public void setMoveAble(boolean moveAble){
        this.moveAble = moveAble;
    }
    /**
     * Tells whether this pokemon can attack.
     * <br /><br />
     * Diz se este pokémon pode atacar.
     * 
     * @return true if this pokemon can attack
     */
    public boolean isAttackAble(){
        return this.attackAble;
    }
    /**
     * Defines whether this pokemon can attack.
     * <br /><br />
     * Define se este pokémon pode atacar.
     * 
     * @param attackAble 
     *          a boolean value indicating if this pokemon can attack
     */
    public void setAttackAble(boolean attackAble){
        this.attackAble = attackAble;
    }
    /**
     * Tells if this pokemon is asleep.
     * <br /><br />
     * Diz se este pokémon está adormecido.
     * 
     * @return true if this pokemon is asleep
     */
    public boolean isAsleep(){
        return this.asleep;
    }
    /**
     * Defines if this pokemon is asleep.
     * <br /><br />
     * Define se este pokémon está adormecido.
     * 
     * @param asleep 
     *          a boolean value indicating if this pokemon is asleep
     */
    public void setAsleep(boolean asleep){
        this.asleep = asleep;
        //TODO make a special condition marker. Fazer um msrcador de condição especial para exibir no ActionFrame.
    }
    public boolean isBurned(){
        return this.burned;
    }
    public void setBurned(boolean burned){
        this.burned = burned;
        //TODO make a special condition marker.
    }
    public boolean isConfused(){
        return this.confused;
    }
    public void setConfused(boolean confused){
        this.confused = confused;
        //TODO maka special condition marker.
    }
    public boolean isParalyzed(){
        return this.paralyzed;
    }
    public void setParalyzed(boolean paralyzed){
        this.paralyzed = paralyzed;
        //TODO maka special condition marker.
    }
    /**
     * Tells if this pokemon is poisoned.
     * <br /><br />
     * Diz se este pokémon está envenenado.
     * 
     * @return true if this pokemon is poisoned
     */
    public boolean isPoisoned(){
        return this.poisoned;
    }
    /**
     * Defines if this pokemon is poisoned.
     * <br /><br />
     * Define se este pokémon está envenenado.
     * 
     * @param poisoned 
     *          a boolean value indicating if this pokemon is poisoned
     */
    public void setPoisoned(boolean poisoned){
        this.poisoned = poisoned;
        //TODO maka special condition marker.
    }
    /**
     * Tells if this pokemon is immune to damage.
     * <br /><br />
     * Diz se este pokémon está imune a dano.
     * 
     * @return true if this pokemon is poisoned
     */
    public boolean isDamageImmune(){
        return this.damageImmune;
    }
    /**
     * Defines if this pokemon is immune to damage.
     * <br /><br />
     * Define se este pokémon está imune a dano.
     * 
     * @param damageImmune 
     *          a boolean value indicating if this pokemon is immune to damage
     */
    public void setDamageImmune(boolean damageImmune){
        this.damageImmune = damageImmune;
        //TODO maka special condition marker.
    }
    /**
     * Tells if this pokemon was knocked out.
     * <br /><br />
     * Diz se este pokémon foi nocauteado.
     * 
     * @return true 
     *          if this pokemon was knocked out
     */
    public boolean isKnockedOut(){
        return this.knockedOut;
    }
    
    //Action methods
    
    /**
     * Apply damage to the pokemon considering weakness and resistance.
     * Also checks whether the pokemon go on knock out. If so, removes it from the board.
     * <br /><br />
     * Aplica dano ao pokémon considerando fraquesa e resistência.
     * Também confere se o pokémon irá a nocaute. Caso for, o remove do tabuleiro.
     * 
     * @param damage 
     *          the amount of damage received
     * @param attackType 
     *          the attack element type
     */
    public void setDamageTaken(int damage, PokemonType attackType){
        if(!damageImmune){
            //Considering weakness and resistance.
            if(this.weakness == attackType){
                damage *= 2;
            } else if(this.resistance == attackType){
                damage -= 3;
            }
            this.hitPoints -= (damage > 0) ? damage : 0;
            //Check for knock out.
            if(this.hitPoints <= 0){
                //Pokemon knocked out.
                knockedOut = true;
                Greenfoot.playSound("Pokemon Critical Stop.wav");
                World board = getWorld();
                board.removeObject(this);
            }
        }
    }
    
    /**
     * Heals a pokemon for a determined amount of damage.
     * <br /><br />
     * Cura um pokémon de uma determinada quantidade de dano.
     * 
     * @param heal 
     *          the amount of damage to be healed
     */
    public void setDamageHealed(int heal){
        if((this.hitPoints + heal) <= this.maxHitPoints){
            this.hitPoints += heal;
        }
    }
    
    /**
     * Shows the places where this pokemon can go.
     * Add various MoveField to the world to show the squares where this pokemon can move through.
     * <br /><br />
     * Mostra os lugares onde este pokémon pode ir.
     * Adiciona varios MoveField ao mundo para mostrar as casas onde este pokémon pode se mover.
     */
    protected void showMoveField(){
        //TODO improve this method to match original game rules.
        //Implementar o bloqueio de movimento.
        int cAtMt;//The number of cells the pokémon can move through (cells Able to Move through).
        cAtMt = 4 - this.retreatCost;
        //
        int[][] boardArray = new int[8][8];
        //0 for possible to move on.
        //1 for not possible to move on.
        
        int pokeX = this.getX();//Pokemon's current X coordinates.
        int pokeY = this.getY();//Pokemon's current Y coordinates.
        for(int x=0; x < boardArray.length; x++){
            for(int y=0; y < boardArray[x].length; y++){
                //Initialy the pokemon can not move.
                boardArray[x][y] = 1;
            }
        }
        World board = getWorld();
        for(int x=1; x <= cAtMt; x++){
            for(int y=1; y <= cAtMt; y++){
                int rightX = pokeX + x;   //We're going right side the board.
                int leftX = pokeX - x;    //We're going left side the board.
                int downY = pokeY + y;    //We're going down the board.
                int upY = pokeY - y;      //We're going up the board.
                
                //Test for outside border and marking cells able to move on.
                if(rightX < board.getWidth()){
                    //
                    boardArray[rightX][pokeY] = 0;
                    if(downY < board.getHeight()){
                        //
                        if(x < cAtMt && y < cAtMt){
                            boardArray[rightX][downY] = 0;
                        }
                        if(cAtMt > 2 && (x == cAtMt - 1 && y > 1) || (y == cAtMt - 1 && x > 1)){
                            boardArray[rightX][downY] = 1;
                        }
                    }
                    if(upY >= 0){
                        //
                        if(x < cAtMt && y < cAtMt){
                            boardArray[rightX][upY] = 0;
                        }
                        if(cAtMt > 2 && (x == cAtMt - 1 && y > 1) || (y == cAtMt - 1 && x > 1)){
                            boardArray[rightX][upY] = 1;
                        }
                    }
                }
                if(leftX >= 0){
                    //
                    boardArray[leftX][pokeY] = 0;
                    if(downY < board.getHeight()){
                        //
                        if(x < cAtMt && y < cAtMt){
                            //
                            boardArray[leftX][downY] = 0;
                        }
                        if(cAtMt > 2 && (x == cAtMt - 1 && y > 1) || (y == cAtMt - 1 && x > 1)){
                            boardArray[leftX][downY] = 1;
                        }
                    }
                    if(upY >= 0){
                        //
                        if(x < cAtMt && y < cAtMt){
                            boardArray[leftX][upY] = 0;
                        }
                        if(cAtMt > 2 && (x == cAtMt - 1 && y > 1) || (y == cAtMt - 1 && x > 1)){
                            boardArray[leftX][upY] = 1;
                        }
                    }
                }
                if(upY >= 0){
                    //
                    boardArray[pokeX][upY] = 0;
                }
                if(downY < board.getHeight()){
                    //
                    boardArray[pokeX][downY] = 0;
                }
            }
        }
        List neighbours = this.getNeighbours(cAtMt, false, Pokemon.class);
        for(int n=0; n < neighbours.size(); n++){
            Pokemon pk = (Pokemon) neighbours.get(n);
            boardArray[pk.getX()][pk.getY()] = 1;
        }
        
        //To finish, a pokemon can not move on itself.
        boardArray[pokeX][pokeY] = 1;
        
        //Print the move fields on the PokeChessBoard.
        for(int i=0; i < boardArray.length; i++){
            for(int j=0; j < boardArray[i].length; j++){
                if(boardArray[i][j] == 0){
                    board.addObject(new MoveField(), i, j);
                }
            }
        }
    }
    
    /**
     * Place this pokemon somewhere.
     * <br /><br />
     * Coloca este pokémon em algum lugar no tabuleiro.
     * 
     * @param x 
     *          the x coordinate
     * @param y 
     *          the y coordinate
     */
    public void move(int x, int y){
        //Avoiding a pokemon moving on another one.
        Pokemon targetPokemon = (Pokemon) getOneObjectAtOffset(x, y, Pokemon.class);
        if(targetPokemon == null){
            setLocation(x, y);
            this.moveAble = false;
        }
    }
    
    /**
     * Shows the range of an attack which doesn't have a specific target.
     * Add an AttackField on this pokemon x,y coordinate.
     * <br /><br />
     * Mostra o alcance de um ataque que não possui um alvo específico.
     * Adiciona um AttackField na coordenada x,y deste pokémon.
     */
    protected void showSelfAttackRange(){
        World board = getWorld();
        board.addObject(new AttackField(), this.getX(), this.getY());
    }
    
    /**
     * Shows the range of a melee attack.
     * Add four AttackField around this pokemon making a + sign in the board.
     * <br /><br />
     * Mostra o alcance de um ataque corpo-a-corpo.
     * Adiciona quatro AttackField ao redor deste pokémon formando um sinal de + no tabuleiro.
     */
    protected void showMeleeAttackRange(){
        int pokeX = this.getX();//Pokemon's current X coordinates.
        int pokeY = this.getY();//Pokemon's current Y coordinates.
        World board = getWorld();
        if(pokeX + 1 < board.getWidth()){
            board.addObject(new AttackField(), pokeX + 1, pokeY);
        }
        if(pokeX - 1 >= 0){
            board.addObject(new AttackField(), pokeX - 1, pokeY);
        }
        if(pokeY + 1 < board.getHeight()){
            board.addObject(new AttackField(), pokeX, pokeY + 1);
        }
        if(pokeY - 1 >= 0){
            board.addObject(new AttackField(), pokeX, pokeY - 1);
        }
    }
    
    /**
     * Shows the range of a ranged attack.
     * A ranged attack's range is calculated as follows.
     * For each colorless energy required to perform the attack it's range increase in one square on the board, 
     * and for each distinct energy required to perform the attack it's range increase in two squares on the board.
     * i.e.: Consider Magmar's flame thrower attack, which only can be performed with two fire energies and a colorless energy.
     * The straigth forward range is: a colorless energy = one square + two fire energies = four squares, 
     * which sums to a five square range.
     * The diagonal range is obtained in the same way, however the final value is divided by two and 
     * rounded down in case of non integer result.
     * The result looks like the way the queen moves on a chess board. Reguarding the attack's range.
     * <br /><br />
     * Mostra o alcance de um ataque a distância.
     * O alcance de um ataque a distância é calculado da seguinte forma:
     * Para cada energia incolor necessária para executar o ataque o seu alcance aumenta em uma casa no tabuleiro, 
     * e para cada energia distinta necessária para executar o ataque o seu alcance aumenta em duas casas no tabuleiro.
     * Ex: Considere o ataque lança-chamas do Magmar, que somente pode ser executado com duas energias de fogo e uma incolor. 
     * O alcance em linha reta será de: uma energia incolor = uma casa + duas energias de fogo = quatro casas, 
     * somando um total de cinco casas de alcance.
     * O alcance em diagonal é calculado da mesma forma que o alcance em linha reta, porém o valor final é dividido por dois e 
     * arredondado para baixo no caso de resultado não inteiro. Que neste caso seria: 5 / 2 = 2,5 ≃ 2.
     * O resultado final se assemelha à forma com que uma dama se movimenta em um tabuleiro de xadrez. Com a diferença de 
     * respeitar o alcance máximo do ataque.
     * 
     * @param range 
     *          the range in squares
     */
    protected void showRangedAttackRange(int range){
        //TODO Implement block system. Implementar o sistema de bloqueio.
        int[][] boardArray = new int[8][8];
        //0 = not in attack's range.
        //1 = in attack's range.
        
        //Initialy the pokemon has no attack range.
        //Inicialmente não é considerado alcance de ataque para o pokémon.
        /*
         * Como estamos sinalizando "fora do alcance" com zero, não há necessidade de alterar os valores 
         * do array boardArray, pois variáveis do tipo int são sempre inicializadas em zero (mesmo os elementos de um array de int).
         */
        
        int pokeX = this.getX();//Pokemon's current X coordinates.
        int pokeY = this.getY();//Pokemon's current Y coordinates.
        
        World board = getWorld();
        int x = 1;
        int y = 1;
        while(x <= range) {
            int rightX = pokeX + x;   //We're going right side the board.
            int leftX = pokeX - x;    //We're going left side the board.
            int downY = pokeY + y;    //We're going down the board.
            int upY = pokeY - y;      //We're going up the board.
            
            if(rightX < board.getWidth()){
                boardArray[rightX][pokeY] = 1;
                if(downY < board.getHeight()){
                    if(x <= range/2){
                       boardArray[rightX][downY] = 1;
                    }
                }
                if(upY >= 0){
                    if(x <= range/2){
                        boardArray[rightX][upY] = 1;
                    }
                }
            }
            if(leftX >= 0){
                boardArray[leftX][pokeY] = 1;
                if(downY < board.getHeight()){
                    if(x <= range/2){
                        boardArray[leftX][downY] = 1;
                    }
                }
                if(upY >= 0){
                    if(x <= range/2){
                        boardArray[leftX][upY] = 1;
                    }
                }
            }
            if(upY >= 0){
                boardArray[pokeX][upY] = 1;
            }
            if(downY < board.getHeight()){
                boardArray[pokeX][downY] = 1;
            }
            x++;
            y++;
        }
        
        //Print the attack fields on the PokeChessBoard.
        for(int i=0; i < boardArray.length; i++){
            for(int j=0; j < boardArray[i].length; j++){
                if(boardArray[i][j] == 1){
                    board.addObject(new AttackField(), i, j);
                }
            }
        }
    }
    
    /**
     * Shows the range of an attack by area.
     * The range of an attack by area is calculated by the same way of a ranged attack:
     * For each colorless energy required to perform the attack it's range increase in one square on the board, 
     * and for each distinct energy required to perform the attack it's range increase in two squares on the board.
     * The filed of range is made by every square reachable by the following way:
     * Starting from the attacking pokemon square, it's inside of range every square which is connected to the starting
     * square by one of it's edges. This process is repeated the number of times equal to the attack's range.
     * <br /><br />
     * Mostra o alcance de um ataque por área.
     * O alcance de um ataque por área é calculado da mesma forma que o ataque a distância:
     * Para cada energia incolor necessária para executar o ataque o seu alcance aumenta em uma casa no tabuleiro, 
     * e para cada energia distinta necessária para executar o ataque o seu alcance aumenta em duas casas no tabuleiro.
     * O campo de alcance é formado por todos as casas atingiveis da seguinte forma:
     * Partindo da casa onde o pokémon atacante está, considera-se dentro do alcance toda a casa que está ligada à casa de 
     * partida atraves de uma aresta. Este processo se repete por número de vezes igual ao alcance do ataque.
     * 
     * @param range 
     *          the range in squares
     * 
     * @see showRangedAttackRange
     */
    protected void showByAreaAttackRange(int range) {
        //TODO o alcanse do ataque não é representado corretamente para ataques com alcance maior ou igual a 7.
        int[][] boardArray = new int[8][8];
        //0 = not in attack's range.
        //1 = in attack's range.
        
        int pokeX = this.getX();//Pokemon's current X coordinates.
        int pokeY = this.getY();//Pokemon's current Y coordinates.
        World board = getWorld();
        for(int x=1; x <= range; x++){
            for(int y=1; y <= range; y++){
                int rightX = pokeX + x;   //We're going right side the board.
                int leftX = pokeX - x;    //We're going left side the board.
                int downY = pokeY + y;    //We're going down the board.
                int upY = pokeY - y;      //We're going up the board.
                
                if(rightX < board.getWidth()){
                    boardArray[rightX][pokeY] = 1;
                    if(downY < board.getHeight()){
                        if(x < range && y < range){
                            boardArray[rightX][downY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[rightX][downY] = 0;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[rightX][downY] = 0;
                        }
                    }
                    if(upY >= 0){
                        if(x < range && y < range){
                            boardArray[rightX][upY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[rightX][upY] = 0;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[rightX][upY] = 0;
                        }
                    }
                }
                if(leftX >= 0){
                    boardArray[leftX][pokeY] = 1;
                    if(downY < board.getHeight()){
                        if(x < range && y < range){
                            boardArray[leftX][downY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[leftX][downY] = 0;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[leftX][downY] = 0;
                        }
                    }
                    if(upY >= 0){
                        if(x < range && y < range){
                            boardArray[leftX][upY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[leftX][upY] = 0;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[leftX][upY] = 0;
                        }
                    }
                }
                if(upY >= 0){
                    boardArray[pokeX][upY] = 1;
                }
                if(downY < board.getHeight()){
                    boardArray[pokeX][downY] = 1;
                }
            }
        }
        
        //Print the attack fields on the PokeChessBoard.
        for(int i=0; i < boardArray.length; i++){
            for(int j=0; j < boardArray[i].length; j++){
                if(boardArray[i][j] == 1){
                    board.addObject(new AttackField(), i, j);
                }
            }
        }
    }
    
    /**
     * Shows the range of an attack in area.
     * The range of an attack in area is calculated in the same way of an attack by area.
     * O targets of an attack in area are all pokemon inside it's range.
     * <br /><br />
     * Mostra o alcance de um ataque em área.
     * O alcance do ataque em área é calculado da mesma forma que o alcance de um ataque por área.
     * Os alvos de um ataque em área são todos os pokémons que estão dentro de seu alcance.
     * 
     * @param range 
     *          the range in squares
     * 
     * @see showByAreaAttackRange
     * @see showRangedAttackRange
     */
    protected void showInAreaAttackRange(int range){
        int[][] boardArray = new int[8][8];
        //0 = not in attack's range.
        //1 = in attack's range.
        
        int pokeX = this.getX();//Pokemon's current X coordinates.
        int pokeY = this.getY();//Pokemon's current Y coordinates.
        World board = getWorld();
        for(int x=1; x <= range; x++){
            for(int y=1; y <= range; y++){
                int rightX = pokeX + x;   //We're going right side the board.
                int leftX = pokeX - x;    //We're going left side the board.
                int downY = pokeY + y;    //We're going down the board.
                int upY = pokeY - y;      //We're going up the board.
                
                if(rightX < board.getWidth()){
                    boardArray[rightX][pokeY] = 1;
                    if(downY < board.getHeight()){
                        if(x < range && y < range){
                            boardArray[rightX][downY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[rightX][downY] = 1;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[rightX][downY] = 1;
                        }
                    }
                    if(upY >= 0){
                        if(x < range && y < range){
                            boardArray[rightX][upY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[rightX][upY] = 1;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[rightX][upY] = 1;
                        }
                    }
                }
                if(leftX >= 0){
                    boardArray[leftX][pokeY] = 1;
                    if(downY < board.getHeight()){
                        if(x < range && y < range){
                            boardArray[leftX][downY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[leftX][downY] = 1;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[leftX][downY] = 1;
                        }
                    }
                    if(upY >= 0){
                        if(x < range && y < range){
                            boardArray[leftX][upY] = 1;
                        }
                        if(range > 2 && (x == range - 1 && y > 1) || (y == range - 1 && x > 1)){
                            boardArray[leftX][upY] = 1;
                        }
                        if(range > 3 && (x == range - 2 && y > 2) || (y == range - 2 && x > 2)){
                            boardArray[leftX][upY] = 1;
                        }
                    }
                }
                if(upY >= 0){
                    boardArray[pokeX][upY] = 1;
                }
                if(downY < board.getHeight()){
                    boardArray[pokeX][downY] = 1;
                }
            }
        }
        
        //Print the attack fields on the PokeChessBoard.
        for(int i=0; i < boardArray.length; i++){
            for(int j=0; j < boardArray[i].length; j++){
                if(boardArray[i][j] == 1){
                    board.addObject(new AttackField(), i, j);
                }
            }
        }
    }
    
    /**
     * Act - do whatever the Pokemon wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        // Add your action code here.
    }    
}